@@ -1,11 +1,12 @@
 use std::ffi::CString;
-use std::{io, slice};
+use std::io;
 
 use hdfs_sys::*;
 use log::debug;
 
-use crate::file_info::FileInfo;
-use crate::stream_builder::StreamBuilder;
+use crate::file::OpenOptions;
+use crate::metadata::Metadata;
+use crate::Readdir;
 
 /// Client holds the underlying connection to hdfs clusters.
 ///
@@ -34,6 +35,10 @@ pub struct Client {
     fs: hdfsFS,
 }
 
+/// HDFS's client handle is thread safe.
+unsafe impl Send for Client {}
+unsafe impl Sync for Client {}
+
 impl Client {
     /// Connect to a name node with port
     ///
@@ -70,67 +75,87 @@ impl Client {
     /// use hdrs::Client;
     ///
     /// let fs = Client::connect("default").expect("client connect succeed");
-    /// let builder = fs.open("/tmp/hello.txt", libc::O_RDONLY);
+    /// let open_options = fs.open_file();
     /// ```
-    pub fn open(&self, path: &str, flags: i32) -> io::Result<StreamBuilder> {
-        debug!("open file {} with flags {}", path, flags);
-        let b = unsafe {
-            let p = CString::new(path)?;
-            hdfsStreamBuilderAlloc(self.fs, p.as_ptr(), flags)
-        };
-
-        if b.is_null() {
-            return Err(io::Error::last_os_error());
-        }
-
-        debug!("file {} with flags {} opened", path, flags);
-        Ok(StreamBuilder::new(self.fs, b))
+    pub fn open_file(&self) -> OpenOptions {
+        OpenOptions::new(self.fs)
     }
 
-    /// Delete a dir or directory.
-    ///
-    /// - If the path is a file, recursive will have no effect.
-    /// - If the path is a dir and recursive is `true`, the dir will be deleted.
-    /// - If the path is an empty dir and recursive is `false`, the dir will be deleted.
-    /// - If the path is non-empty dir and recursive is `false`, an error will be raised.
-    ///
-    /// # Todo
-    ///
-    /// - We need to handle the `Directory /tmp/xxx is not empty` error.
-    /// - Do we need to split this function into `remove_file` and `remove_dir`?
+    /// Delete a file.
     ///
     /// # Examples
     ///
-    /// ## Delete a file
-    ///
     /// ```
     /// use hdrs::Client;
     ///
     /// let fs = Client::connect("default").expect("client connect succeed");
-    /// let _ = fs.delete("/tmp/hello.txt", false);
+    /// let _ = fs.remove_file("/tmp/hello.txt");
     /// ```
-    ///
-    /// ## Delete a directory
-    ///
-    /// ```
-    /// use hdrs::Client;
-    ///
-    /// let fs = Client::connect("default").expect("client connect succeed");
-    /// let _ = fs.delete("/tmp/hello", true);
-    /// ```
-    pub fn delete(&self, path: &str, recursive: bool) -> io::Result<()> {
-        debug!("delete path {} with recursive {}", path, recursive);
+    pub fn remove_file(&self, path: &str) -> io::Result<()> {
+        debug!("remove file {}", path);
 
         let n = unsafe {
             let p = CString::new(path)?;
-            hdfsDelete(self.fs, p.as_ptr(), recursive.into())
+            hdfsDelete(self.fs, p.as_ptr(), false.into())
         };
 
         if n == -1 {
             return Err(io::Error::last_os_error());
         }
 
-        debug!("delete path {} with recursive {} finished", path, recursive);
+        debug!("delete file {} finished", path);
+        Ok(())
+    }
+
+    /// Delete a dir.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use hdrs::Client;
+    ///
+    /// let fs = Client::connect("default").expect("client connect succeed");
+    /// let _ = fs.remove_dir("/tmp/xxx");
+    /// ```
+    pub fn remove_dir(&self, path: &str) -> io::Result<()> {
+        debug!("remove dir {}", path);
+
+        let n = unsafe {
+            let p = CString::new(path)?;
+            hdfsDelete(self.fs, p.as_ptr(), false.into())
+        };
+
+        if n == -1 {
+            return Err(io::Error::last_os_error());
+        }
+
+        debug!("delete dir {} finished", path);
+        Ok(())
+    }
+
+    /// Delete a dir recursively.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use hdrs::Client;
+    ///
+    /// let fs = Client::connect("default").expect("client connect succeed");
+    /// let _ = fs.remove_dir_all("/tmp/xxx/");
+    /// ```
+    pub fn remove_dir_all(&self, path: &str) -> io::Result<()> {
+        debug!("remove dir all {}", path);
+
+        let n = unsafe {
+            let p = CString::new(path)?;
+            hdfsDelete(self.fs, p.as_ptr(), true.into())
+        };
+
+        if n == -1 {
+            return Err(io::Error::last_os_error());
+        }
+
+        debug!("delete dir all {} finished", path);
         Ok(())
     }
 
@@ -144,7 +169,7 @@ impl Client {
     /// use hdrs::Client;
     ///
     /// let fs = Client::connect("default").expect("client connect succeed");
-    /// let fi = fs.stat("/tmp/hello.txt");
+    /// let fi = fs.metadata("/tmp/hello.txt");
     /// ```
     ///
     /// ## Stat a non-exist path
@@ -155,11 +180,11 @@ impl Client {
     /// use hdrs::Client;
     ///
     /// let fs = Client::connect("default").expect("client connect succeed");
-    /// let fi = fs.stat("/tmp/not-exist.txt");
+    /// let fi = fs.metadata("/tmp/not-exist.txt");
     /// assert!(fi.is_err());
     /// assert_eq!(fi.unwrap_err().kind(), io::ErrorKind::NotFound)
     /// ```
-    pub fn stat(&self, path: &str) -> io::Result<FileInfo> {
+    pub fn metadata(&self, path: &str) -> io::Result<Metadata> {
         let hfi = unsafe {
             let p = CString::new(path)?;
             hdfsGetPathInfo(self.fs, p.as_ptr())
@@ -170,11 +195,9 @@ impl Client {
         }
 
         // Safety: hfi must be valid
-        let fi = unsafe { FileInfo::try_from(*hfi)? };
+        let fi = unsafe { Metadata::from(*hfi) };
 
         // Make sure hfi has been freed.
-        //
-        // FIXME: do we need to free file info if `FileInfo::try_from` failed?
         unsafe { hdfsFreeFileInfo(hfi, 1) };
 
         Ok(fi)
@@ -188,9 +211,9 @@ impl Client {
     /// use hdrs::Client;
     ///
     /// let fs = Client::connect("default").expect("client connect succeed");
-    /// let fis = fs.readdir("/tmp/hello/");
+    /// let fis = fs.read_dir("/tmp/hello/");
     /// ```
-    pub fn readdir(&self, path: &str) -> io::Result<Vec<FileInfo>> {
+    pub fn read_dir(&self, path: &str) -> io::Result<Readdir> {
         let mut entries = 0;
         let hfis = unsafe {
             let p = CString::new(path)?;
@@ -206,25 +229,49 @@ impl Client {
             let e = io::Error::last_os_error();
 
             return match e.raw_os_error() {
-                None => Ok(Vec::new()),
-                Some(code) if code == 0 => Ok(Vec::new()),
+                None => Ok(Vec::new().into()),
+                Some(code) if code == 0 => Ok(Vec::new().into()),
                 Some(_) => Err(e),
             };
         }
 
         let mut fis = Vec::with_capacity(entries as usize);
 
-        let hf = unsafe { slice::from_raw_parts(hfis, entries as usize) };
-        for v in hf {
-            fis.push(FileInfo::try_from(*v)?)
+        for i in 0..entries {
+            let m = unsafe { Metadata::from(*hfis.offset(i as isize)) };
+
+            fis.push(m)
         }
 
         // Make sure hfis has been freed.
-        //
-        // FIXME: do we need to free file info if `FileInfo::try_from` failed?
         unsafe { hdfsFreeFileInfo(hfis, entries) };
 
-        Ok(fis)
+        Ok(fis.into())
+    }
+
+    /// mkdir create dir and all it's parent directories.
+    ///
+    /// The behavior is similar to `mkdir -p /path/to/dir`.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use hdrs::Client;
+    ///
+    /// let fs = Client::connect("default").expect("client connect succeed");
+    /// let _ = fs.create_dir("/tmp");
+    /// ```
+    pub fn create_dir(&self, path: &str) -> io::Result<()> {
+        let n = unsafe {
+            let p = CString::new(path)?;
+            hdfsCreateDirectory(self.fs, p.as_ptr())
+        };
+
+        if n == -1 {
+            return Err(io::Error::last_os_error());
+        }
+
+        Ok(())
     }
 }
 
@@ -252,9 +299,7 @@ mod tests {
 
         let path = uuid::Uuid::new_v4().to_string();
 
-        let _ = fs
-            .open(&format!("/tmp/{path}"), libc::O_RDONLY)
-            .expect("open file success");
+        let _ = fs.open_file().read(true).open(&format!("/tmp/{path}"));
     }
 
     #[test]
@@ -266,7 +311,7 @@ mod tests {
 
         let path = uuid::Uuid::new_v4().to_string();
 
-        let f = fs.stat(&format!("/tmp/{path}"));
+        let f = fs.metadata(&format!("/tmp/{path}"));
         assert!(f.is_err());
         assert_eq!(f.unwrap_err().kind(), io::ErrorKind::NotFound);
     }
@@ -278,7 +323,20 @@ mod tests {
         let fs = Client::connect("default").expect("init success");
         debug!("Client: {:?}", fs);
 
-        let f = fs.readdir("/tmp").expect("open file success");
-        assert!(!f.is_empty())
+        let f = fs.read_dir("/tmp").expect("open file success");
+        debug!("Metadata: {:?}", f);
+        assert!(f.len() > 0)
+    }
+
+    #[test]
+    fn test_client_mkdir() {
+        let _ = env_logger::try_init();
+
+        let fs = Client::connect("default").expect("init success");
+        debug!("Client: {:?}", fs);
+
+        let _ = fs
+            .create_dir("/tmp")
+            .expect("mkdir on exist dir should succeed");
     }
 }
