@@ -10,19 +10,22 @@
 //! # fn main() -> Result<(), Box<dyn std::error::Error>> {
 //! let fs = Client::connect("default")?;
 //!
-//! let mut f = fs
-//!     .open("/tmp/hello.txt", libc::O_WRONLY | libc::O_CREAT)?
-//!     .build()?;
+//! let mut f = fs.open_file().write(true).create(true).open("/tmp/hello.txt")?;
 //! let n = f.write("Hello, World!".as_bytes())?;
 //!
-//! let mut f = fs.open("/tmp/hello.txt", libc::O_RDONLY)?.build()?;
+//! let mut f = fs.open_file().read(true).open("/tmp/hello.txt")?;
 //! let mut buf = vec![0; 1024];
 //! let n = f.read(&mut buf)?;
 //!
-//! let _ = fs.delete("/tmp/hello.txt", false)?;
+//! let _ = fs.remove_file("/tmp/hello.txt")?;
 //! # Ok(())
 //! # }
 //! ```
+//!
+//! # Features
+//!
+//! - `futures-io`: Enable [`futures`](https://docs.rs/futures/latest/futures/io/index.html) support for [`File`]
+//! - `tokio-io`: Enable [`tokio::io`](https://docs.rs/tokio/latest/tokio/io/index.html) support for [`File`]
 
 mod client;
 pub use client::Client;
@@ -30,5 +33,8 @@ pub use client::Client;
 mod file;
 pub use file::File;
 
-mod file_info;
-mod stream_builder;
+mod metadata;
+pub use metadata::Metadata;
+
+mod readdir;
+pub use readdir::Readdir;
